#!/bin/bash

export PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:$PATH

V_ID="$1"
P_ID="$2"


if [ -z "$V_ID" ] || [ -z "$P_ID" ]; then
    exit 0
fi

NAME=realtek-usb-wifi
PACKAGE_NAME=$NAME-dkms
DEB_NAME=$(echo $PACKAGE_NAME | sed 's,_,-,')
ARCH=`dpkg-architecture -qDEB_BUILD_GNU_CPU`
CURRENT_KERNEL=$(uname -r)

sed -i '/CONFIG_RTL8822BU=m/d' /lib/modules/$CURRENT_KERNEL/build/include/config/auto.conf
sed -i '/CONFIG_RTL8821CU=m/d' /lib/modules/$CURRENT_KERNEL/build/include/config/auto.conf
sed -i '/CONFIG_RTL8192FU=m/d' /lib/modules/$CURRENT_KERNEL/build/include/config/auto.conf

function notify-send() {
    #Detect the name of the display in use
    local display=":$(ls /tmp/.X11-unix/* | sed 's#/tmp/.X11-unix/X##' | head -n 1)"

    #Detect the user using such display
    local user=$(who | grep '('$display')' | awk '{print $1}' | head -n 1)

    #Detect the id of the user
    local uid=$(id -u $user)

    sudo -u $user DISPLAY=$display DBUS_SESSION_BUS_ADDRESS=unix:path=/run/user/$uid/bus notify-send "$@"
}

loger() {
    echo "$NAME: $1"
    logger -t $NAME "$1"
}

notifier() {
	notify-send -a $NAME $@
}

dkms_conf_proc() {
	#loger "$1 dkms conf process..."
	
	#cp -rf $2 $2.bak
	#sed -i '$ a\'$new_add $2
	module_name=$(grep 'BUILT_MODULE_NAME\[0\]' "$2" | awk -F'"' '{print $2}')
	sed -i "s/$module_name/$1/g" $2
}

loger "Device($V_ID:$P_ID) insert..."
#loger "Running user is: $(whoami) $(logname)"

#
lsusb -d $V_ID:$P_ID > /dev/null
if [ $? -eq 0 ]; then
    loger "找到匹配的设备：VID=$V_ID PID=$P_ID"
else
    loger "没有找到匹配的设备：VID=$V_ID PID=$P_ID"
fi

unset CONFIG_RTL8822BU
unset CONFIG_RTL8822CU
unset CONFIG_RTL8852BU
unset CONFIG_RTL8188EU
unset CONFIG_RTL8192FU
unset CONFIG_RTL8723DU
unset CONFIG_RTL8821CU
unset CONFIG_RTL8851BU
unset CONFIG_RTL8188FU
unset CONFIG_RTL8852CU

#
if ([ $V_ID == "3574" ] && [ $P_ID == "6241" ]) \
|| ([ $V_ID == "3574" ] && [ $P_ID == "6221" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "a85b" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "b85a" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "b852" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "b83a" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "b832" ]); then

	DRIVER=8852bu
	export CONFIG_RTL8852BU=m

elif ([ $V_ID == "3574" ] && [ $P_ID == "4211" ]) \
|| ([ $V_ID == "07b8" ] && [ $P_ID == "8179" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "0179" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "8179" ]); then

	DRIVER=8188eu
	export CONFIG_RTL8188EU=m

elif ([ $V_ID == "3574" ] && [ $P_ID == "4231" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "f179" ]); then

	DRIVER=8188fu
	export CONFIG_RTL8188FU=m

elif ([ $V_ID == "3574" ] && [ $P_ID == "4261" ]) \
|| ([ $V_ID == "0b05" ] && [ $P_ID == "18f1" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "a725" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "f192" ]); then

	DRIVER=8192fu
	export CONFIG_RTL8192FU=m

elif ([ $V_ID == "3574" ] && [ $P_ID == "4251" ]) \
|| ([ $V_ID == "7392" ] && [ $P_ID == "d611" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "d723" ]); then

	DRIVER=8723du
	export CONFIG_RTL8723DU=m

elif ([ $V_ID == "3574" ] && [ $P_ID == "5231" ]) \
|| ([ $V_ID == "3574" ] && [ $P_ID == "5211" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "c80c" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "8731" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "8811" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "c811" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "c82b" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "c82a" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "c820" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "c821" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "b820" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "b82b" ]); then

	DRIVER=8821cu
	export CONFIG_RTL8821CU=m

elif ([ $V_ID == "3574" ] && [ $P_ID == "6231" ]) \
|| ([ $V_ID == "3574" ] && [ $P_ID == "6211" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "b851" ]); then

	DRIVER=8851bu
	export CONFIG_RTL8851BU=m

elif ([ $V_ID == "2357" ] && [ $P_ID == "0151" ]) \
|| ([ $V_ID == "2357" ] && [ $P_ID == "0150" ]) \
|| ([ $V_ID == "2357" ] && [ $P_ID == "014d" ]) \
|| ([ $V_ID == "2357" ] && [ $P_ID == "6251" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "c832" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "c85d" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "c832" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "c85a" ]); then

	DRIVER=8852cu
	export CONFIG_RTL8852CU=m

elif ([ $V_ID == "3574" ] && [ $P_ID == "5261" ]) \
|| ([ $V_ID == "3574" ] && [ $P_ID == "5241" ]) \
|| ([ $V_ID == "13b1" ] && [ $P_ID == "0043" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "b812" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "b82c" ]); then

	DRIVER=88x2bu
	export CONFIG_RTL8822BU=m

elif ([ $V_ID == "2357" ] && [ $P_ID == "0137" ]) \
|| ([ $V_ID == "3574" ] && [ $P_ID == "5251" ]) \
|| ([ $V_ID == "13b1" ] && [ $P_ID == "0043" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "d82b" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "d820" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "c812" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "c82e" ]) \
|| ([ $V_ID == "0bda" ] && [ $P_ID == "c82c" ]); then

	DRIVER=88x2cu
	export CONFIG_RTL8822CU=m
else
    loger "is Not Realtek USB WiFi Device!"
	exit 0;
fi

loger "Device($V_ID:$P_ID) is $DRIVER!"

#
#/lib/modules/$CURRENT_KERNEL/
modinfo $DRIVER > /dev/null
if [ $? -eq 0 ] ; then
	loger "$DRIVER($V_ID:$P_ID) driver is ready !"
	exit 0
fi

loger "==> $DRIVER($V_ID:$P_ID) driver for $CURRENT_KERNEL-$ARCH"


#
dpkg -l | grep $PACKAGE_NAME > /dev/null
if [ $? -ne 0 ] ; then
	loger "Please install $PACKAGE_NAME package !"
	notifier "系统未安装 $PACKAGE_NAME 驱动包，请安装！"
	exit 0
fi

VERSION=`dpkg-query -W -f='${Version}' $DEB_NAME | awk -F "-" '{print $1}' | cut -d\: -f2`

#
dkms_status=`dkms status -m $NAME -v $VERSION`
if [ `echo $dkms_status | grep -c ": built"` -eq 1 ] ||  
	[ `echo $dkms_status | grep -c ": installed"` -eq 1 ]; then

	loger "Removing old $NAME-$VERSION DKMS files..."
	dkms remove -m $NAME -v $VERSION -k $CURRENT_KERNEL > /dev/null
	#rm -rf /var/lib/dkms/$NAME/$VERSION/$CURRENT_KERNEL/$ARCH

	if [ -d "/usr/src/$NAME-$VERSION" ]; then
		loger "Loading new $NAME-$VERSION DKMS files..."
		dkms add -m $NAME -v $VERSION > /dev/null
	fi
	
elif [ `echo $dkms_status | grep -c ": added"` -eq 1 ]; then
	loger "Source is ready!"
else
	loger "Please install $PACKAGE_NAME  package !"
	notifier "系统未安装 $PACKAGE_NAME 驱动包，请安装！"
	exit 0	
fi


#
loger "Start build $DRIVER($V_ID:$P_ID) driver $NAME-$VERSION ..."
notifier "USB-WIFI驱动正在安装中，请稍后..." "大约1~2分钟！"

#dkms_conf_proc $DRIVER /usr/src/$NAME-$VERSION/dkms.conf
dkms_conf_proc $DRIVER /var/lib/dkms/$NAME/$VERSION/source/dkms.conf

dkms build -m $NAME -v $VERSION -k $CURRENT_KERNEL
if [ $? -eq 0 ] ; then
	loger "Build $DRIVER($V_ID:$P_ID) driver $NAME-$VERSION susecss!"
else
	loger "Build $DRIVER($V_ID:$P_ID) driver $NAME-$VERSION failed!"
	exit $?
fi

##
dkms_status=`dkms status -m $NAME -v $VERSION -k $CURRENT_KERNEL` 
if [ `echo $dkms_status | grep -c ": built"` -eq 1 ] && 
	[ `echo $dkms_status | grep -c ": installed"` -eq 0 ]; then
	
	loger "Start install $DRIVER($V_ID:$P_ID) driver $NAME-$VERSION ..."
	dkms install -m $NAME -v $VERSION -k $CURRENT_KERNEL --force
	if [ $? -eq 0 ] ; then
		loger "Install $DRIVER($V_ID:$P_ID) driver susecss!"
		notifier "USB-WIFI驱动已安装完成！"
	else
		loger "Install $DRIVER($V_ID:$P_ID) driver failed!"
		notifier "USB-WIFI驱动安装失败，请重试！"
	fi
fi

exit 0;